package com.example.nto.service.impl;

import com.example.nto.mapper.EmployeeDTOMapper;
import com.example.nto.model.EmployeeRoleType;
import com.example.nto.model.dto.EmployeeDTO;
import com.example.nto.model.entity.Employee;
import com.example.nto.repository.CodeRepository;
import com.example.nto.repository.EmployeeRepository;
import com.example.nto.service.EmployeeService;
import com.example.nto.service.exception.CodeNotFoundException;
import com.example.nto.service.exception.EmployeeBannedException;
import com.example.nto.service.exception.EmployeeNotFoundException;
import com.example.nto.service.exception.NotAnAdminException;
import lombok.RequiredArgsConstructor;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.Objects;

@Service
@RequiredArgsConstructor
public class EmployeeServiceImpl implements EmployeeService {

    private final EmployeeRepository employeeRepository;

    private final PasswordEncoder passwordEncoder;

    private final CodeRepository codeRepository;

    private final EmployeeDTOMapper employeeDTOMapper;

    private void employeeExists(final String login) {
        if (!employeeRepository.existsByLogin(login)) {
            throw new EmployeeNotFoundException();
        }
    }

    @Override
    public EmployeeDTO getEmployeeDTO(final String login, final String recipientLogin) {
        employeeExists(login);
        var e = employeeRepository.findEmployeeByLogin(login);
        if (!Objects.equals(e.getLogin(), recipientLogin)) {
            var recipient = getEmployee(recipientLogin);
            if (recipient.getRole() != EmployeeRoleType.ADMIN) {
                throw new NotAnAdminException();
            }
        }
        return employeeDTOMapper.map(e);
    }

    private Employee getEmployee(final String login) {
        employeeExists(login);
        return employeeRepository.findEmployeeByLogin(login);
    }

    @Override
    public void updateVisit(final String login, final long value) {
        employeeExists(login);
        if (!codeRepository.existsByValue(value)) {
            throw new CodeNotFoundException();
        }
        final Employee employee = getEmployee(login);
        if (employee.getIsBanned()) {
            throw new EmployeeBannedException();
        }
        employee.setLastVisit(LocalDateTime.now());
        employeeRepository.save(employee);
    }

    @Override
    public void addEmployee(Employee employee) {
        employee.setPassword(passwordEncoder.encode(employee.getPassword()));
        employeeRepository.save(employee);
    }

    @Override
    public void banEmployee(String login) {
        employeeExists(login);
        var employee = getEmployee(login);
        employee.setIsBanned(true);
        employeeRepository.save(employee);
    }

    @Override
    public void unbanEmployee(String login) {
        employeeExists(login);
        var employee = getEmployee(login);
        employee.setIsBanned(false);
        employeeRepository.save(employee);
    }
}
